from enum import Enum


class Provider(Enum):
    ANTHROPIC = "ANTHROPIC"
    API4AI = "API4AI"
    AWS = "AWS"
    AZURE = "AZURE"
    CLARIFAI = "CLARIFAI"
    GCP = "GCP"
    MODERNMT = "MODERNMT"
    OPENAI = "OPENAI"
    SENTISIGHT = "SENTISIGHT"
